summaryrefslogtreecommitdiffhomepage
path: root/Types.hs
diff options
context:
space:
mode:
authorJoey Hess <joeyh@joeyh.name>2017-05-02 15:52:27 -0400
committerJoey Hess <joeyh@joeyh.name>2017-05-02 17:01:35 -0400
commitf559fcfadd7079140ed64bab68275527f46d334e (patch)
tree1f30f563093a27188a5b1da37aa764f4e58c0393 /Types.hs
parent9456361ed8f6dd094a4c08cc352f9a1fd9d0069f (diff)
downloaddebug-me-f559fcfadd7079140ed64bab68275527f46d334e.tar.gz
add prevEntered pointer
Client requires this always point to the previous Entered it accepted, so a hash chain of Entered is built up, and there is no possibility for ambiguity about which order a client received two Entered activies in. So restoreHashes now has to try every possible combination of known hashes for both prevEntered and prevActivity. That could be significantly more work, but it would be unusual for there to be a lot of known hashes, so it should be ok. --graphviz shows this additional hash chain with grey edges (and leaves out edges identical to the other hash chain) While testing this with an artifical network lag, it turned out that signature verification was failing for Reject messages sent by the user. Didn't quite figure out what was at the bottom of that, but the Activity Entered that was sent back in a Reject message was clearly not useful, because it probably had both its prevEntered and prevActivity hashes set to Nothing (because restoreHashes didn't restore them, because the original Activity Entered was out of the expected chain). So, switched Rejected to use a Hash. (And renamed Rejected to EnteredRejected to make it more clear what it's rejecting.) Also, added a lastAccepted hash to EnteredRejected. This lets the developer find its way back to the accepted chain when some of its input gets rejected. This commit was sponsored by Trenton Cronholm on Patreon.
Diffstat (limited to 'Types.hs')
-rw-r--r--Types.hs14
1 files changed, 11 insertions, 3 deletions
diff --git a/Types.hs b/Types.hs
index 233a7de..8f00793 100644
--- a/Types.hs
+++ b/Types.hs
@@ -65,6 +65,9 @@ instance DataSize a => DataSize (Message a) where
data Activity a = Activity
{ activity :: a
, prevActivity :: Maybe Hash
+ -- ^ Pointer to previous activity Seen/Entered
+ , prevEntered :: Maybe Hash
+ -- ^ Pointer to previous activity Entered
, elapsedTime :: ElapsedTime
, activitySignature :: Signature
}
@@ -92,8 +95,12 @@ instance DataSize Control where
+ dataSize (controlSignature c)
data ControlAction
- = Rejected (Activity Entered)
- -- ^ sent by user to indicate when an Entered value was rejected.
+ = EnteredRejected
+ { enteredRejected :: Hash
+ -- ^ Entered value that was rejected.
+ , enteredLastAccepted :: Maybe Hash
+ -- ^ The last Entered value that was accepted.
+ }
| SessionKey (PerhapsSigned PublicKey)
-- ^ sent by user at start, and later by developer,
-- to indicate their session key
@@ -108,7 +115,8 @@ data ControlAction
type SenderName = Val
instance DataSize ControlAction where
- dataSize (Rejected a) = dataSize a
+ dataSize (EnteredRejected h1 h2) = dataSize h1 +
+ maybe 0 dataSize h2
dataSize (SessionKey k) = dataSize k
dataSize (SessionKeyAccepted k) = dataSize k
dataSize (SessionKeyRejected k) = dataSize k